# Orchid Information Manager
# This shows aging information for a customer.
# Field 1 should be the Customer Number field.
from accpac import *
import eb
import datetime
import locale
import time

transactions = None
p1 = 0
p2 = 0
p3 = 0

# init is called when the script is first loaded.
def init(arg):
	pass

def accpacDateToDate(d):
	if d == "0":
		return None
	return datetime.date(int(d[0:4]), int(d[4:6]), int(d[6:8]))

def onRefreshContent():
	global p1
	global p2
	global p3
	locale.setlocale(locale.LC_ALL, "")
	eb.clearAll()

	now = time.strftime("%Y%m%d")
	v = openView("AR0055", 1) # ARAGE

	vOPT = openView("AR0004", 1) # ARR04
	vOPT.recordClear()
	vOPT.browse()
	vOPT.fetch()
	p1 = vOPT.get("AGINPERD1")
	p2 = vOPT.get("AGINPERD2")
	p3 = vOPT.get("AGINPERD3")

	v.recordClear()
	v.put("CMNDCODE", 61, False) # Single Age Calculation
	v.put("AGEINVDTSW", 1, False) # Document Date
	v.put("CUSTID", eb.value1, False)
	v.put("RUNDATE", now, False)
	v.put("CUTOFFDATE", "20991231", False)
	v.put("SWPREPAYMT", 1, False) # Include Prepayments
	v.put("AGEPERIOD2", vOPT.get("AGINPERD1"), False) # first period
	v.put("AGEPERIOD3", vOPT.get("AGINPERD2"), False) # second period
	v.put("AGEPERIOD4", vOPT.get("AGINPERD3"), False) # third period
	v.process()

	g = eb.addGrid()
	g.name = "grid"
	g.button1 = "Open"
	col = g.addColumn("From " + eb.value1)
	col = g.addColumn("To")
	col = g.addColumn("Amount Due")
	col.align = "RIGHT"

	r = g.addRow()
	r.setCell(0, "Current")
	r.setCell(1, "")
	r.setCell(2, "{:,.2f}".format(v.get("TOTFWDTC") + v.get("AMTDUE1TC")))
	r.setCellColor(0, RGB(0,0,255))

	r = g.addRow()
	r.setCell(0, "1")
	r.setCell(1, "{:,.0f}".format(p1))
	r.setCell(2, "{:,.2f}".format(v.get("AMTDUE2TC")))
	r.setCellColor(0, RGB(0,0,255))

	r = g.addRow()
	r.setCell(0, "{:,.0f}".format(p1+1))
	r.setCell(1, "{:,.0f}".format(p2))
	r.setCell(2, "{:,.2f}".format(v.get("AMTDUE3TC")))
	r.setCellColor(0, RGB(0,0,255))

	r = g.addRow()
	r.setCell(0, "{:,.0f}".format(p2+1))
	r.setCell(1, "{:,.0f}".format(p3))
	r.setCell(2, "{:,.2f}".format(v.get("AMTDUE4TC")))
	r.setCellColor(0, RGB(0,0,255))

	r = g.addRow()
	r.setCell(0, "Over")
	r.setCell(1, "{:,.0f}".format(p3))
	r.setCell(2, "{:,.2f}".format(v.get("AMTDUE5TC")))
	r.setCellColor(0, RGB(0,0,255))

# Called when you double click / press enter on an item in the grid
def onAction_Open(ctrlName, rowIndex):
	rowIndex = int(rowIndex)
	if ctrlName == "gridTransactions":
		transactions[rowIndex].drilldown()
		return
	displayTransactions(rowIndex)

# Button1 is the 'Open' button. It opens AR Customers.
def onAction_Button1(ctrlName):
	if ctrlName == "gridTransactions":
		# it's the Back button
		onRefreshContent()
		return
	openUI("AR1300", "IDCUST=" + eb.value1, False)

def displayTransactions(n):
	global transactions

	eb.clearAll()

	now = time.strftime("%Y%m%d")
	g = eb.addGrid()
	g.name = "gridTransactions"
	g.button1 = "Back"
	col = g.addColumn("Transaction Type")
	col = g.addColumn("Document No.")
	col = g.addColumn("Transaction Date")
	col = g.addColumn("Amount Due")
	col.align = "RIGHT"
	transactions = []

	if n == 0: # Current
		startDate = datetime.datetime.now()
		startDate = startDate.strftime("%Y%m%d")
		endDate = 99993112
	elif n== 1: # 1
		startDate = datetime.datetime.now() + datetime.timedelta(days=0-p1)
		endDate = datetime.datetime.now()
		startDate = startDate.strftime("%Y%m%d")
		endDate = endDate.strftime("%Y%m%d")
	elif n== 2: # 2
		startDate = datetime.datetime.now() + datetime.timedelta(days=0-p2)
		endDate = datetime.datetime.now() + datetime.timedelta(days=0-p1)
		startDate = startDate.strftime("%Y%m%d")
		endDate = endDate.strftime("%Y%m%d")
	elif n== 3: # 3
		startDate = datetime.datetime.now() + datetime.timedelta(days=0-p3)
		endDate = datetime.datetime.now() + datetime.timedelta(days=0-p2)
		startDate = startDate.strftime("%Y%m%d")
		endDate = endDate.strftime("%Y%m%d")
	elif n== 4: # Over
		startDate = 0
		endDate = datetime.datetime.now() + datetime.timedelta(days=0-p3)
		endDate = endDate.strftime("%Y%m%d")

	arobl = openView("AR0036", 1)
	arobl.order(7) # IDCUST, DATEINVC
	arobl.recordClear()
	n = 0
	arobl.browse("IDCUST=\"" + eb.value1 + "\" AND SWPAID=0 AND DATEDUE>=" + str(startDate) + " AND DATEDUE<" + str(endDate))
	while arobl.fetch() == 0:
		n = n + 1
		r = g.addRow()
		tx = arobl.get("TRXTYPETXT")
		if tx == 1:
			r.setCell(0, "Invoice")
		elif tx == 2:
			r.setCell(0, "Debit Note")
		elif tx == 3:
			r.setCell(0, "Credit Note")
		elif tx == 4:
			r.setCell(0, "Interest")
		elif tx == 5:
			r.setCell(0, "Unapplied Cash")
		elif tx == 10:
			r.setCell(0, "Prepayment")
		elif tx == 11:
			r.setCell(0, "Receipt")
		elif tx == 19:
			r.setCell(0, "Refund")
		else:
			r.setCell(0, str(arobl.get("TRXTYPETXT")))
		r.setCellColor(0, RGB(0,0,255))
		r.setCell(1, arobl.get("IDINVC"))
		r.setCell(2, accpacDateToDate(arobl.get("DATEINVC")).strftime("%x"))
		r.setCell(3, "{:,.2f}".format(arobl.get("AMTDUETC")))
		tran = ARTransaction()
		tran.tx = tx
		tran.batch = arobl.get("CNTBTCH")
		tran.entry = arobl.get("CNTITEM")
		transactions.append(tran)

class ARTransaction():
	def __init__(self):
		self.tx = 0
		self.batch = None
		self.entry = None

	def drilldown(self):
		if self.tx in [1,2,3,4]:
			# open AR Invoice Entry
			openUI("AR2100", "BATCH=" + str(self.batch) + "\nENTRY=" + str(self.entry) + "\nMODE=1\n", False)
		elif self.tx in [11,10,5]:
			# open AR Receipt Entry
			openUI("AR3100", "BATCH=" + str(self.batch) + "\nENTRY=" + str(self.entry) + "\nMODE=1\n", False)
		elif self.tx in [19]:
			#1400
			# open AR Refund Entry
			openUI("AR5020", "BATCH=" + str(self.batch) + "\nENTRY=" + str(self.entry) + "\nMODE=1\n", False)
